@@ -3,13 +3,13 @@ import numpy as np
 from pydicom.tag import Tag
 import SimpleITK as sitk
 from skimage import draw
-from scipy.ndimage.morphology import binary_fill_holes
 from skimage.measure import label,regionprops,find_contours
 from threading import Thread
 from multiprocessing import cpu_count
 from queue import *
 import pandas as pd
 import copy
+import cv2 as cv
 import matplotlib.pyplot as plt
 
 
@@ -103,21 +103,19 @@ def worker_def(A):
 
 
 class Point_Output_Maker_Class(object):
-    def __init__(self, image_size_rows, image_size_cols, slice_info, PixelSize, mult1, mult2,
-                 ShiftRows, ShiftCols, ShiftZ, contour_dict, mv, shift_list, RS, ds):
+    def __init__(self, image_size_rows, image_size_cols, slice_info, PixelSize, contour_dict, mv, shift_list,
+                 RS, ds):
         self.image_size_rows, self.image_size_cols = image_size_rows, image_size_cols
         self.slice_info = slice_info
         self.PixelSize = PixelSize
-        self.ShiftRows, self.ShiftCols, self.ShiftZ = ShiftRows, ShiftCols, ShiftZ
-        self.mult1, self.mult2 = mult1, mult2
         self.mv = mv
         self.shift_list = shift_list
         self.contour_dict = contour_dict
         self.RS = RS
         self.ds = ds
+        self.shift_list = shift_list
 
     def make_output(self, annotation, i):
-        Xx, Xy, Xz, Yx, Yy, Yz = self.mv
         self.contour_dict[i] = []
         regions = regionprops(label(annotation))
         for ii in range(len(regions)):
@@ -129,41 +127,31 @@ class Point_Output_Maker_Class(object):
                 rows.append(data[iii][0])
                 cols.append(data[iii][1])
             temp_image[rows, cols] = 1
-            points = find_contours(temp_image, 0)[0]
-            output = []
-            for point in points:
-                output.append(((point[1]) * self.PixelSize[0] + self.mult1[i] * self.ShiftRows[i]))
-                output.append(self.mult2[i] * ((point[0]) * self.PixelSize[1] + self.ShiftCols[i]))
-                output.append(self.ShiftZ[i] + point[1] * self.PixelSize[0] * Xz + point[0] * self.PixelSize[1] * Yz)
-            self.contour_dict[i].append(output)
-        hole_annotation = 1 - annotation
-        filled_annotation = binary_fill_holes(annotation)
-        hole_annotation[filled_annotation == 0] = 0
-        regions = regionprops(label(hole_annotation))
-        for ii in range(len(regions)):
-            temp_image = np.zeros([self.image_size_rows, self.image_size_cols])
-            data = regions[ii].coords
-            rows = []
-            cols = []
-            for iii in range(len(data)):
-                rows.append(data[iii][0])
-                cols.append(data[iii][1])
-            temp_image[rows, cols] = 1
-            points = find_contours(temp_image, 0)[0]
-            output = []
-            for point in points:
-                output.append(((point[1]) * self.PixelSize[0] + self.mult1[i] * self.ShiftRows[i]))
-                output.append(((point[0]) * self.PixelSize[1] + self.mult2[i] * self.ShiftCols[i]))
-                output.append(self.ShiftZ[i] + point[1] * self.PixelSize[0] * Xz + point[0] * self.PixelSize[1] * Yz)
-            self.contour_dict[i].append(output)
+            contours = find_contours(temp_image, 0)
+            for contour in contours:
+                self.contour_dict[i].append(contour)
 
 
 class DicomReaderWriter:
     def __init__(self, rewrite_RT_file=False, delete_previous_rois=True,Contour_Names=None,
-                 template_dir=None, channels=3, get_images_mask=True, arg_max=True,
-                 associations={},desc='',iteration=0, get_dose_output=False, **kwargs):
+                 template_dir=None, get_images_mask=True, arg_max=True,
+                 associations={},desc='',iteration=0, get_dose_output=False, flip_axes=(False, False, False), **kwargs):
+        '''
+        :param rewrite_RT_file: Boolean, should we re-write the RT structure
+        :param delete_previous_rois: delete the previous RTs within the structure
+        :param Contour_Names: list of contour nmes
+        :param template_dir: default to None, specifies path to template RT structure
+        :param get_images_mask: boolean, load the images and mask
+        :param arg_max: perform argmax on the mask
+        :param associations: dictionary of associations {'liver_bma_program_4': 'liver'}
+        :param desc: description information to add to .nii files
+        :param iteration: what iteration for writing .nii files
+        :param get_dose_output: boolean, collect dose information
+        :param flip_axes: tuple(3), axis that you want to flip, defaults to (False, False, False)
+        :param kwargs:
+        '''
         self.get_dose_output = get_dose_output
-        self.flip_axes = [False, False, False]
+        self.flip_axes = flip_axes
         self.associations = associations
         self.set_contour_names(Contour_Names)
         self.set_associations(associations)
@@ -178,7 +166,6 @@ class DicomReaderWriter:
         self.template_dir = template_dir
         self.template = True
         self.delete_previous_rois = delete_previous_rois
-        self.channels = channels
         self.get_images_mask = get_images_mask
         self.reader = sitk.ImageSeriesReader()
         self.reader.MetaDataDictionaryArrayUpdateOn()
@@ -364,8 +351,9 @@ class DicomReaderWriter:
         self.RTs_in_case[self.lstRSFile] = rois_in_structure
 
     def get_mask(self):
-        self.mask = np.zeros([len(self.dicom_names), self.image_size_rows, self.image_size_cols, len(self.Contour_Names) + 1],
-                             dtype='int8')
+        self.mask = np.zeros(
+            [len(self.dicom_names), self.image_size_rows, self.image_size_cols, len(self.Contour_Names) + 1],
+            dtype='int8')
         for RT_key in self.RTs_in_case:
             found_rois = {}
             ROIName_Number = self.RTs_in_case[RT_key]
@@ -390,11 +378,11 @@ class DicomReaderWriter:
                     mask = self.Contours_to_mask(index)
                     self.mask[..., self.Contour_Names.index(ROI_Name) + 1][mask == 1] = 1
         if self.flip_axes[0]:
-            self.mask = self.mask[::-1, ...]
+            self.mask = self.mask[:, :, ::-1, ...]
         if self.flip_axes[1]:
             self.mask = self.mask[:, ::-1, ...]
         if self.flip_axes[2]:
-            self.mask = self.mask[..., ::-1]
+            self.mask = self.mask[::-1, ...]
         if self.arg_max:
             self.mask = np.argmax(self.mask, axis=-1)
         self.annotation_handle = sitk.GetImageFromArray(self.mask.astype('int8'))
@@ -403,36 +391,20 @@ class DicomReaderWriter:
         self.annotation_handle.SetDirection(self.dicom_handle.GetDirection())
         return None
 
-    def Contours_to_mask(self, i):
+    def Contours_to_mask(self, index):
         mask = np.zeros([len(self.dicom_names), self.image_size_rows, self.image_size_cols], dtype='int8')
-        Contour_data = self.RS_struct.ROIContourSequence[i].ContourSequence
-        shifts = [[float(i) for i in self.reader.GetMetaData(j, "0020|0032").split('\\')] for j in range(len(self.reader.GetFileNames()))]
-        Xx, Xy, Xz, Yx, Yy, Yz = [float(i) for i in self.reader.GetMetaData(0, "0020|0037").split('\\')]
-        PixelSize = self.dicom_handle.GetSpacing()
-
+        Contour_data = self.RS_struct.ROIContourSequence[index].ContourSequence
         for i in range(len(Contour_data)):
-            referenced_sop_instance_uid = Contour_data[i].ContourImageSequence[0].ReferencedSOPInstanceUID
-            if referenced_sop_instance_uid not in self.SOPInstanceUIDs:
-                print('{} Error here with instance UID'.format(self.PathDicom))
-                return None
-            else:
-                slice_index = self.SOPInstanceUIDs.index(referenced_sop_instance_uid)
-            ShiftRowsBase, ShiftColsBase, ShiftzBase = shifts[slice_index]
-            mult1 = 1
-            mult2 = 1
-            if ShiftRowsBase > 0:
-                mult1 = -1
-            if ShiftColsBase > 0:
-                mult2 = -1
-            ShiftRows = ShiftRowsBase * Xx + ShiftColsBase * Xy + ShiftzBase * Xz
-            ShiftCols = ShiftRowsBase * Xy + ShiftColsBase * Yy + ShiftzBase * Yz
-
-            rows = Contour_data[i].ContourData[1::3]
-            cols = Contour_data[i].ContourData[0::3]
-            row_val = [abs(x - mult1 * ShiftRows)/PixelSize[0] for x in cols]
-            col_val = [abs(x - mult2 * ShiftCols)/PixelSize[1] for x in rows]
-            temp_mask = self.poly2mask(col_val, row_val, [self.image_size_rows, self.image_size_cols])
-            mask[slice_index, :, :][temp_mask > 0] += 1
+            as_array = np.asarray(Contour_data[i].ContourData[:])
+            reshaped = np.reshape(as_array, [as_array.shape[0]//3, 3])
+            matrix_points = np.asarray([self.dicom_handle.TransformPhysicalPointToIndex(reshaped[i])
+                                        for i in range(reshaped.shape[0])])
+            self.col_val = matrix_points[:, 0]
+            self.row_val = matrix_points[:, 1]
+            z_vals = matrix_points[:, 2]
+            temp_mask = self.poly2mask(self.row_val, self.col_val, [self.image_size_rows, self.image_size_cols])
+            temp_mask[self.row_val, self.col_val] = 0
+            mask[z_vals[0], temp_mask] += 1
         mask = mask % 2
         return mask
 
@@ -450,26 +422,25 @@ class DicomReaderWriter:
 
     def get_images(self):
         self.dicom_handle = self.reader.Execute()
+        self.mv = [float(i) for i in self.reader.GetMetaData(0, "0020|0037").split('\\')]
+        self.shift_list = [[float(i) for i in self.reader.GetMetaData(j, "0020|0032").split('\\')]
+                           for j in range(len(self.reader.GetFileNames()))] #ShiftRows, ShiftCols, ShiftZBase
+        # self.shift_list = [self.dicom_handle[:, :, i:i+1].GetOrigin() for i in range(self.dicom_handle.GetSize()[-1])]
         sop_instance_UID_key = "0008|0018"
         self.SOPInstanceUIDs = [self.reader.GetMetaData(i, sop_instance_UID_key) for i in
                                 range(self.dicom_handle.GetDepth())]
         slice_location_key = "0020|0032"
         self.slice_info = [self.reader.GetMetaData(i, slice_location_key).split('\\')[-1] for i in
                            range(self.dicom_handle.GetDepth())]
-        imageorientation_patient_key = "0020|0037"
-        flipimagefilter = sitk.FlipImageFilter()
-        self.image_orientation_patient = [float(i) for i in
-                                          self.reader.GetMetaData(0, imageorientation_patient_key).split('\\')][3:]
-        for i in range(3):
-            if self.image_orientation_patient[i] == -1.0:
-                self.flip_axes[i] = True
-            else:
-                self.flip_axes[i] = False
-        flipimagefilter.SetFlipAxes(self.flip_axes)
-        self.dicom_handle = flipimagefilter.Execute(self.dicom_handle)
+        self.patient_position = self.reader.GetMetaData(0, "0018|5100")
+        if max(self.flip_axes):
+            flipimagefilter = sitk.FlipImageFilter()
+            flipimagefilter.SetFlipAxes(self.flip_axes)
+            self.dicom_handle = flipimagefilter.Execute(self.dicom_handle)
         self.ArrayDicom = sitk.GetArrayFromImage(self.dicom_handle)
         self.image_size_cols, self.image_size_rows, self.image_size_z = self.dicom_handle.GetSize()
 
+
     def write_images_annotations(self, out_path):
         image_path = os.path.join(out_path, 'Overall_Data_{}_{}.nii.gz'.format(self.desciption, self.iteration))
         annotation_path = os.path.join(out_path, 'Overall_mask_{}_y{}.nii.gz'.format(self.desciption,self.iteration))
@@ -514,27 +485,17 @@ class DicomReaderWriter:
         self.output_dir = output_dir
         if len(annotations.shape) == 3:
             annotations = np.expand_dims(annotations, axis=-1)
+        if self.flip_axes[0]:
+            annotations = annotations[:, :, ::-1, ...]
+        if self.flip_axes[1]:
+            annotations = annotations[:, ::-1, ...]
+        if self.flip_axes[2]:
+            annotations = annotations[::-1, ...]
         self.annotations = annotations
         self.Mask_to_Contours()
 
     def Mask_to_Contours(self):
         self.RefDs = self.ds
-        self.shift_list = [[float(i) for i in self.reader.GetMetaData(j, "0020|0032").split('\\')] for j in range(len(self.reader.GetFileNames()))] #ShiftRows, ShiftCols, ShiftZBase
-        self.mv = Xx, Xy, Xz, Yx, Yy, Yz = [float(i) for i in self.reader.GetMetaData(0, "0020|0037").split('\\')]
-        self.ShiftRows = [i[0] * Xx + i[1] * Xy + i[2] * Xz for i in self.shift_list]
-        self.ShiftCols = [i[0] * Xy + i[1] * Yy + i[2] * Yz for i in self.shift_list]
-        self.ShiftZ = [i[2] for i in self.shift_list]
-        self.mult1 = []
-        self.mult2 = []
-        for i in self.shift_list:
-            if i[0] > 0:
-                self.mult1.append(-1)
-            else:
-                self.mult1.append(1)
-            if i[1] > 0:
-                self.mult2.append(-1)
-            else:
-                self.mult2.append(1)
         self.PixelSize = self.dicom_handle.GetSpacing()
         current_names = []
         for names in self.RS_struct.StructureSetROISequence:
@@ -544,7 +505,6 @@ class DicomReaderWriter:
         for name in self.ROI_Names:
             Contour_Key[name] = xxx
             xxx += 1
-        self.all_annotations = self.annotations
         base_annotations = copy.deepcopy(self.annotations)
         temp_color_list = []
         color_list = [[128, 0, 0], [170, 110, 40], [0, 128, 128], [0, 0, 128], [230, 25, 75], [225, 225, 25],
@@ -558,19 +518,15 @@ class DicomReaderWriter:
                 temp_color_list = copy.deepcopy(color_list)
             color_int = np.random.randint(len(temp_color_list))
             print('Writing data for ' + Name)
-            self.annotations = copy.deepcopy(base_annotations[:, :, :, int(self.ROI_Names.index(Name) + 1)])
-            self.annotations = self.annotations.astype('int')
-            if self.flip_axes[0]:
-                self.annotations = self.annotations[::-1, ...]
-            if self.flip_axes[1]:
-                self.annotations = self.annotations[:, ::-1, ...]
-            if self.flip_axes[2]:
-                self.annotations = self.annotations[..., ::-1]
+            annotations = copy.deepcopy(base_annotations[:, :, :, int(self.ROI_Names.index(Name) + 1)])
+            annotations = annotations.astype('int')
 
             make_new = 1
             allow_slip_in = True
             if (Name not in current_names and allow_slip_in) or self.delete_previous_rois:
                 self.RS_struct.StructureSetROISequence.insert(0,copy.deepcopy(self.RS_struct.StructureSetROISequence[0]))
+                # self.RS_struct.ROIContourSequence[0].ContourSequence[0].ContourData = []
+                # self.RS_struct.ROIContourSequence[0].ContourSequence[0].NumberofContourPoints = 0
             else:
                 print('Prediction ROI {} is already within RT structure'.format(Name))
                 continue
@@ -597,47 +553,51 @@ class DicomReaderWriter:
             self.RS_struct.ROIContourSequence[self.struct_index].ROIDisplayColor = temp_color_list[color_int]
             del temp_color_list[color_int]
             thread_count = int(cpu_count()*0.9-1)
-            thread_count = 1
             contour_dict = {}
             q = Queue(maxsize=thread_count)
             threads = []
             kwargs = {'image_size_rows': self.image_size_rows, 'image_size_cols': self.image_size_cols,
-                      'slice_info': self.slice_info, 'PixelSize': self.PixelSize, 'mult1': self.mult1,
-                      'mult2': self.mult2, 'ShiftZ': self.ShiftZ, 'mv': self.mv, 'shift_list': self.shift_list,
-                      'ShiftRows': self.ShiftRows, 'ShiftCols': self.ShiftCols, 'contour_dict': contour_dict, 'RS': self.RS_struct,
-                      'ds': self.ds}
+                      'slice_info': self.slice_info, 'PixelSize': self.PixelSize, 'mv': self.mv,
+                      'shift_list': self.shift_list, 'contour_dict': contour_dict,
+                      'RS': self.RS_struct, 'ds': self.ds}
 
             A = [q,kwargs]
+            # pointer_class = Point_Output_Maker_Class(**kwargs)
             for worker in range(thread_count):
                 t = Thread(target=contour_worker, args=(A,))
                 t.start()
                 threads.append(t)
-
             contour_num = 0
-            if np.max(self.annotations) > 0:  # If we have an annotation, write it
-                image_locations = np.max(self.annotations, axis=(1, 2))
+            if np.max(annotations) > 0:  # If we have an annotation, write it
+                image_locations = np.max(annotations, axis=(1, 2))
                 indexes = np.where(image_locations > 0)[0]
                 for index in indexes:
-                    item = [self.annotations[index, ...], index]
+                    item = [annotations[index, ...], index]
+                    # pointer_class.make_output(*item)
                     q.put(item)
                 for i in range(thread_count):
                     q.put(None)
                 for t in threads:
                     t.join()
                 for i in contour_dict.keys():
-                    for output in contour_dict[i]:
+                    for all_contours in contour_dict[i]:
+                        contours = [[int(contour[1]), int(contour[0]), int(i)] for contour in np.squeeze(all_contours)]
+                        points = [self.dicom_handle.TransformIndexToPhysicalPoint(ii) for ii in contours]
+                        output = np.asarray(points).flatten('C')
                         if contour_num > 0:
                             self.RS_struct.ROIContourSequence[self.struct_index].ContourSequence.append(
                                 copy.deepcopy(
                                     self.RS_struct.ROIContourSequence[self.struct_index].ContourSequence[0]))
                         self.RS_struct.ROIContourSequence[self.struct_index].ContourSequence[
                             contour_num].ContourNumber = str(contour_num)
+                        self.RS_struct.ROIContourSequence[self.struct_index].ContourSequence[
+                            contour_num].ContourGeometricType = 'CLOSED_PLANAR'
                         self.RS_struct.ROIContourSequence[self.struct_index].ContourSequence[
                             contour_num].ContourImageSequence[0].ReferencedSOPInstanceUID = self.SOPInstanceUIDs[i]
                         self.RS_struct.ROIContourSequence[self.struct_index].ContourSequence[
-                            contour_num].ContourData = output
+                            contour_num].ContourData = list(output)
                         self.RS_struct.ROIContourSequence[self.struct_index].ContourSequence[
-                            contour_num].NumberofContourPoints = round(len(output) / 3)
+                            contour_num].NumberofContourPoints = len(output) // 3
                         contour_num += 1
         self.RS_struct.SOPInstanceUID += '.' + str(np.random.randint(999))
         if self.template or self.delete_previous_rois:
@@ -734,8 +694,11 @@ class DicomReaderWriter:
         self.make_array(PathDicom)
         if self.rewrite_RT_file:
             self.rewrite_RT()
-        if not self.template and self.get_images_mask:
-            self.get_mask()
+        if self.get_images_mask and self.Contour_Names is not None:
+            self.mask = np.zeros([len(self.dicom_names), self.image_size_rows, self.image_size_cols, len(self.Contour_Names) + 1],
+                                 dtype='int8')
+            if not self.template:
+                self.get_mask()
         if self.get_dose_output:
             self.get_dose()
         true_rois = []
@@ -784,7 +747,7 @@ class DicomReaderWriter:
 
 class Dicom_to_Imagestack(DicomReaderWriter):
     def __init__(self, **kwargs):
-        print('Please move over to using DicomReaderWriter, same arguments are passed')
+        print('Please move from using Dicom_to_Imagestack to DicomReaderWriter, same arguments are passed')
         super().__init__(**kwargs)
 
 
